==> mongodb-5.0.30.gh.tar.gz: mongo-r5.0.30/src/mongo/db/resource_yielder.h <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/db/operation_context.h"

namespace mongo {
/**
 * Functions to call before and after blocking on the network layer so that resources may be given
 * up so that "sub operations" running on the same node may use them. For example, a node may want
 * to check in its session before waiting on the network so that a sub-operation may check it out
 * and use it. This is important for preventing deadlocks.
 */
class ResourceYielder {
public:
    virtual ~ResourceYielder() = default;

    virtual void yield(OperationContext*) = 0;
    virtual void unyield(OperationContext*) = 0;
};
}  // namespace mongo

==> mongodb-7.0.23.gh.tar.gz: mongo-r7.0.23/src/mongo/db/resource_yielder.h <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/db/operation_context.h"

namespace mongo {
/**
 * Functions to call before and after blocking on the network layer so that resources may be given
 * up so that "sub operations" running on the same node may use them. For example, a node may want
 * to check in its session before waiting on the network so that a sub-operation may check it out
 * and use it. This is important for preventing deadlocks.
 */
class ResourceYielder {
public:
    virtual ~ResourceYielder() = default;

    virtual void yield(OperationContext*) = 0;
    virtual void unyield(OperationContext*) = 0;

    Status yieldNoThrow(OperationContext* opCtx) noexcept {
        try {
            yield(opCtx);
        } catch (const DBException& e) {
            return e.toStatus();
        }
        return Status::OK();
    };

    Status unyieldNoThrow(OperationContext* opCtx) noexcept {
        try {
            unyield(opCtx);
        } catch (const DBException& e) {
            return e.toStatus();
        }
        return Status::OK();
    };
};
}  // namespace mongo
